// Generated by gmmproc 2.54.1 -- DO NOT MODIFY!
#ifndef _GLIBMM_THREAD_P_H
#define _GLIBMM_THREAD_P_H
#ifndef GLIBMM_DISABLE_DEPRECATED


#endif // GLIBMM_DISABLE_DEPRECATED


#endif /* _GLIBMM_THREAD_P_H */

