// Generated by gmmproc 2.54.1 -- DO NOT MODIFY!


#include <glibmm.h>

#include <glibmm/shell.h>
#include <glibmm/private/shell_p.h>


/* Copyright (C) 2002 The gtkmm Development Team
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <glibmm/utility.h>

namespace Glib
{

/**** shell utility functions **********************************************/

Glib::ArrayHandle<std::string>
shell_parse_argv(const std::string& command_line)
{
  char** argv = nullptr;
  int argc = 0;
  GError* error = nullptr;

  g_shell_parse_argv(command_line.c_str(), &argc, &argv, &error);

  if (error)
    Glib::Error::throw_exception(error);

  return Glib::ArrayHandle<std::string>(argv, argc, Glib::OWNERSHIP_DEEP);
}

std::string
shell_quote(const std::string& unquoted_string)
{
  const auto buf = make_unique_ptr_gfree(g_shell_quote(unquoted_string.c_str()));
  return std::string(buf.get());
}

std::string
shell_unquote(const std::string& quoted_string)
{
  GError* error = nullptr;
  char* const buf = g_shell_unquote(quoted_string.c_str(), &error);

  if (error)
    Glib::Error::throw_exception(error);

  return std::string(make_unique_ptr_gfree(buf).get());
}

} // namespace Glib

namespace
{
} // anonymous namespace


Glib::ShellError::ShellError(Glib::ShellError::Code error_code, const Glib::ustring& error_message)
:
  Glib::Error (G_SHELL_ERROR, error_code, error_message)
{}

Glib::ShellError::ShellError(GError* gobject)
:
  Glib::Error (gobject)
{}

Glib::ShellError::Code Glib::ShellError::code() const
{
  return static_cast<Code>(Glib::Error::code());
}

void Glib::ShellError::throw_func(GError* gobject)
{
  throw Glib::ShellError(gobject);
}


